require("scripts/game/login/login_view")
require("scripts/game/login/role_create")
require("scripts/game/login/combine_select_role")
require("scripts/game/login/login_act_code_pop")
require("scripts/game/login/notice_view")
-- 登录
LoginController = LoginController or BaseClass(BaseController)

-- 名称服务器操作返回值定义
TAG_NAME_SERVER_OP_ERROR =
{
	SUCCESS = 0,					--操作成功
	INVALID_NAME = 1,				--名称无效，名称中包含非法字符或长度不合法
	DATA_BASE_ERROR = 2,			--名称服务器数据库操作错误
	DATA_BASE_CALL_ERROR = 3,		--名称服务器数据库调用错误
	NAMEINUSE = 4,					--名称已被使用
	TIME_OUT = 0xFFFF,				--操作超时（本地定义的错误，非名称服务器返回的错误）
}

-- 错误码定义
ERROR_DEFINITION = {
	NOERR					= 0,			--正确
	SQL						= 101,		--sql错误
	SESS					= 102,		--用户没登陆
	GAMESER					= 103,		--游戏服务没准备好
	DATASAVE				= 104,		--角色上一次保存数据是否出现异常
	SELACTOR				= 105,		--客户端选择角色的常规错误
	NOGATE					= 106,		--客户端无匹配的路由数据的错误
	NOUSER					= 107,		--角色不存在
	SEX						= 108,		--错误的性别
	NORANDOMNAME			= 109,		--随机生成的名字已经分配完
	ZY						= 110,		--客户端上传的角色阵营参数错误
	JOB						= 111,		--客户端上传的角色职业参数错误
	GUILD					= 112,		--存在帮派不能删除角色
	SELSERVER				= 113,		--选择服务器错误
	SQL_NOT_CONNECT			= 114,		--sql没有准备好
	PASSWD_ERR				= 115,		--密码错误
	PASSWD_EXIST			= 116,		--密码已经存在
	}

-- 登陆的时候返回的错误码
TAG_LOGIN_OP_ERROR = {
	SUCCESS = 0,	     				--操作成功
	PASSWD_ERROR =1,    				--密码错误
	NOACCOUNT=2,       					--没有这个账号
	IS_ONLINE =3,       				--已经在线
	SERVER_BUSY =4,     				--服务器忙
	SERVER_CLOSE =5,    				--服务器没有开放 
	SESSION_SERVER_ERROR =6 , 			--session服务器有问题，比如db没有连接好
	SERVER_NOT_EXISTING =7, 			--不存在这个服务器
	FCM_LIMITED =8 ,      				--账户纳入防沉迷
	SESSION_SERVER_CLOSE =9, 			--会话服务器处于关闭状态
	DB_SERVER_CLOSE =10,     			--数据服务器处于关闭状态
	LOGIN_IP_LIMIT = 11,     			--登陆IP数量限制

	-- xsp modify begin 新会话服
	IP_ERROR = 12,    					--ip收到了限制
	IP_TOO_MANY_CONNECT = 13,    		--ip连接了太多
	IP_MD5_ERROR = 14,    				--Md5计算错误
	SIGN_OUTOF_DATE = 15,    			--发过来的时间已经过期了
	TIME_FORMAT_ERROR = 16,    			--前面的格式错误
	ACCOUNT_SEAL = 17,    				--登陆过封停的账户
	-- xsp modify end

}

CloseType = 
	{
		CT_ACTORDEL = 0,			 -- 角色被删除
		CT_OTHERLOGIN = 1,			 -- 在别的手机登录
		CT_NORMALLOGIN = 2,		 	 -- 常规被踢下线
		CT_CLIENTOUTTIME = 3,		 -- 客户端心跳包超时
		CT_USEWAIGUA = 4,			 -- 使用外挂
		CT_GMKICKOUT = 5,			 -- GM踢下线
		CT_CROSSLIMIT = 6,           -- 跨服限制
	}

Auto_ReStart_Time = 5 			--自动重连时间

LOGIN_STATE_PLAT_LOGIN = 0
LOGIN_STATE_SERVER_LIST = 1
LOGIN_STATE_CREATE_ROLE = 2
LOGIN_STATE_LOADING = 3
LOGIN_VERIFY_KEY = "566713d23b8810efb313d6934cf77610" 
function LoginController:__init()
	if LoginController.Instance ~= nil then
		ErrorLog("[LoginController] attempt to create singleton twice!")
		return
	end
	LoginController.Instance = self

	self.login_server_heartbeat_timer = nil
	self.game_server_connect_times = 0
	self.loginview = LoginView.New()
	self.createroleview = RoleCreateView.New()
	self.combine_select_role_view = CombineSelectRoleView.New()
	self.act_code_pop = LoginActCodePop.New()

	self.notice_view = NoticeView.New(ViewName.Notice)

	self.disconnect_alert = nil
	self.fail_alert = nil							-- 失败弹出框
	self.enter_gs_count = 0
	self.enter_gs_timer = nil

	self:RegisterAllProtocols()						-- 注册所有协议
	self:RegisterAllEvents()						-- 注册所有需要监听的事件

	self.is_opening_animation_fun = true 	 		-- 是否开启登陆动画功能

	self.is_game_server_connect_success = false
	self.is_login_verifying = false					-- 是否登录验证中

	self.login_state = LOGIN_STATE_PLAT_LOGIN
	require(AGENT_PATH .. "agent_login_view")
	self.agent_login_view = AgentLoginView.New()	-- 渠道相关的登录界面

	AgentMs:InitLimitInfo()
	AgentMs:InitChatLimitInfo(AgentAdapter:GetSpid())
end

function LoginController:__delete()
	LoginController.Instance = nil

	self.loginview:DeleteMe()
	self.loginview = nil

	if nil ~= self.combine_select_role_view then
		self.combine_select_role_view:DeleteMe()
		self.combine_select_role_view = nil
	end

	if nil ~= self.act_code_pop then
		self.act_code_pop:DeleteMe()
		self.act_code_pop = nil
	end

	if nil ~= self.createroleview then
		self.createroleview:DeleteMe()
		self.createroleview = nil
	end
	if nil ~= self.disconnect_alert then
		self.disconnect_alert:DeleteMe()
		self.disconnect_alert = nil
	end

	if nil ~= self.fail_alert then
		self.fail_alert:DeleteMe()
		self.fail_alert = nil
	end

	if nil ~= self.enter_gs_timer then
		GlobalTimerQuest:CancelQuest(self.enter_gs_timer)
		self.enter_gs_timer = nil
	end
	if nil ~= self.busy_alert then
		self.busy_alert = nil
	end

	self.agent_login_view:DeleteMe()
	self.agent_login_view = nil

	self.notice_view:DeleteMe()
	self.notice_view = nil 
end

function LoginController:RegisterAllProtocols()
	self:RegisterProtocol(SCLoginFailAck, "OnLoginFailAck")
	self:RegisterProtocol(SCRoleListAck, "OnRoleListAck")
	self:RegisterProtocol(SCCreateRoleAck, "OnCreateRoleAck")
	self:RegisterProtocol(SCDelRoleAck, "OnDelRoleAck")
	self:RegisterProtocol(SCDisconnectServerNotice, "OnDisconnectServerNotice")
	self:RegisterProtocol(SCCrossServerInfo,"OnCrossServerInfo")
	self:RegisterProtocol(SCQuitCrossServerInfo,"OnQuitCrossServerInfo")
end

function LoginController:RegisterAllEvents()
	GlobalEventSystem:Bind(LoginEventType.GAME_SERVER_CONNECTED, BindTool.Bind(self.OnConnectGameServer, self))
	GlobalEventSystem:Bind(LoginEventType.GAME_SERVER_DISCONNECTED, BindTool.Bind(self.OnDisconnectGameServer, self))
	GlobalEventSystem:Bind(LoginEventType.END_OPENING_ANIMATION, BindTool.Bind(self.EndOpenAnimation, self))
	GlobalEventSystem:Bind(LoginEventType.RECV_MAIN_ROLE_INFO, BindTool.Bind1(self.OnRecvMainRoleInfo, self))
	GlobalEventSystem:Bind(GongGaoEvent.NOTICE_GONGGAO, BindTool.Bind1(self.OnOpenNoticeView, self))
end

function LoginController:StartLogin()
	-- 添加跨服标记
	local cs_info_str = AdapterToLua:getInstance():getDataCache("CROSS_SERVER_INFO")
	local is_reconnect_ing = AdapterToLua:getInstance():getDataCache("IS_RECONNECT_ING")
	if cs_info_str and cs_info_str ~= "" and is_reconnect_ing and is_reconnect_ing == "true" then
		IS_ON_CROSSSERVER = true
	else
		AdapterToLua:getInstance():setDataCache("CROSS_SERVER_INFO", "")
		IS_ON_CROSSSERVER = false
	end
	if (not IS_ON_CROSSSERVER and not self:GetIsReConnectIng()) or not self.loginview:IsLoadedIndex(0) then
		self.loginview:Open()
		if AdapterToLua:getInstance():getDataCache("HAS_OPEN_NOTICE") ~= "true" then
			AdapterToLua:getInstance():setDataCache("HAS_OPEN_NOTICE", "true")
			AgentMs:ReqBoolOpenGongGao()
		end
	end

	-- if not IS_ON_CROSSSERVER or not self:GetIsReConnectIng() then
	-- 	if not MainuiCtrl.Instance:GetView():IsLoadedIndex(0) then
	-- 		MainuiCtrl.Instance:GetView():Release()
	-- 		MainuiCtrl.Instance:GetView():SetVisible()
	-- 		MainuiCtrl.Instance:GetView():Load(0)		-- 主界面加载
	-- 		PerloadCtrl.Instance:Start()
	-- 	end
	-- end

	if not MainuiCtrl.Instance:GetView():IsLoadedIndex(0) then
		MainuiCtrl.Instance:GetView():Release()
		MainuiCtrl.Instance:GetView():SetVisible()
		MainuiCtrl.Instance:GetView():Load(0)		-- 主界面加载
		PerloadCtrl.Instance:Start()
	end

	if self:GetIsReConnectIng() then
		if AgentAdapter.Init then
			AgentAdapter:Init()
		end
		self:Reconnect()
	else
		MainProber:Step(MainProber.STEP_AGENT_LOGIN_BEG)
		self:AgentLoginViewOpen(false)
	end
	Log("StartLogin ... ")
end

function LoginController:AgentLoginViewOpen(only_visible)
	if only_visible then
		if self.agent_login_view:IsLoaded() then
			self.agent_login_view:Open()
			self.login_state = LOGIN_STATE_PLAT_LOGIN
		end
	else
		self.agent_login_view:Open()
		self.login_state = LOGIN_STATE_PLAT_LOGIN
	end
end

function LoginController:AgentLoginViewClose(only_hide)
	if only_hide then
		if self.agent_login_view:IsOpen() then
			self.agent_login_view:CloseVisible()
		end
	else
		self.agent_login_view:Close()
	end
end

function LoginController:AgentLoginViewIsOpen()
	return self.agent_login_view:IsOpen()
end

function LoginController:GetLoginState()
	return self.login_state
end

function LoginController:ResetConnectCount()
	self.game_server_connect_times = 0
end

-- 登录平台成功回调，session_id不为nil则会发送到php后台验证session_id是否合法  muyi5242095
function LoginController:LoginPlatSucc(account_name, session_id)
	
	if self.is_login_verifying then
		return
	end

	MainProber:Step(MainProber.STEP_AGENT_LOGIN_END)
	AdapterToLua:getInstance():setDataCache("PRVE_ACCOUNT_NAME", account_name)

	self.login_state = LOGIN_STATE_SERVER_LIST

	self:LoginVerifyByPhp(account_name, session_id, function(is_succ)
		MainProber:Step(MainProber.STEP_PHP_VERIFY_END or 206, tostring(is_succ))
		if MainProber.Step2 then MainProber:Step2(MainProber.PHP_PUSH_LOGON_END, MainProber.user_id, tostring(is_succ)) end

		if is_succ then
			self.loginview:OnLoginPlatSucc()
		end
	end)
end

-- 通过php验证登录是否合法，取得服务端登录凭证
function LoginController:LoginVerifyByPhp(account_name, session_id, callback_func, times)
	local user_vo = GameVoManager.Instance:GetUserVo()
	user_vo.plat_name = account_name

	if MainProber.Step2 then
		if string.len(account_name) > 4 then
			MainProber.user_id = string.sub(account_name, 5, -1)
		else
			MainProber.user_id = account_name
		end
		MainProber:Step2(MainProber.PHP_PUSH_LOGON_START, MainProber.user_id, GLOBAL_CONFIG.param_list.verify_url)
	end

	if nil ~= session_id and "" ~= session_id and string.len(account_name) > 4 then
		local spid = string.sub(account_name, 1, 3)
		local uid = string.sub(account_name, 5, -1)
		local now_server_time = math.floor(GLOBAL_CONFIG.server_info.server_time + (Status.NowTime - GLOBAL_CONFIG.client_time))

		local try_times = times or 1
		local verify_url = GLOBAL_CONFIG.param_list.verify_url
		if 0 == math.mod(try_times, 2) and nil ~= GLOBAL_CONFIG.param_list.verify_url2 then
			verify_url = GLOBAL_CONFIG.param_list.verify_url2
		end

		local sign = UtilEx:md5Data(spid .. uid .. session_id .. now_server_time .. LOGIN_VERIFY_KEY)
		local real_url = string.format("%s?spid=%s&uid=%s&sid=%s&time=%s&sign=%s&device=%s", 
			verify_url, spid, uid, session_id, now_server_time, sign, tostring(PlatformAdapter.GetPhoneUniqueId()))
		Log("LoginVerifyByPhp, url=" .. real_url)
		user_vo.plat_is_verify = false

		self.is_login_verifying = true
		HttpClient:Request(real_url, "", function(url, arg, data, size)
			Log("LoginVerifyByPhp, callback", url, size, "data:", data)

			if not self.is_login_verifying then
				return
			end

			self.is_login_verifying = false

			if size <= 0 then
				if try_times < 4 then
					self:LoginVerifyByPhp(account_name, session_id, callback_func, try_times + 1)
					return
				end

				if nil ~= callback_func then
					callback_func(false)
				end
				return
			end

			local ret_t = cjson.decode(data)
			if nil == ret_t or 0 ~= ret_t.ret or nil == ret_t.user or ret_t.user.account ~= user_vo.plat_name then
				Log("LoginVerifyByPhp, fail")
				if nil ~= callback_func then
					callback_func(false)
				end
				return
			end

			self:SetLoginUserData(user_vo.plat_name, ret_t.user)

			Log("LoginVerifyByPhp, success")
			if nil ~= callback_func then
				callback_func(true)
			end
		end)
	else
		user_vo.plat_is_verify = true
		if nil ~= callback_func then
			callback_func(true)
		end
	end
end

function LoginController:SetLoginUserData(acount_name, user)
	local user_vo = GameVoManager.Instance:GetUserVo()
	user_vo.plat_is_verify = true
	user_vo.plat_account_type = user.account_type
	user_vo.plat_fcm = user.fcm_flag
	user_vo.plat_login_time = user.login_time
	user_vo.plat_session_key = user.login_sign

	user_vo.plat_name = acount_name

	AdapterToLua:getInstance():setDataCache("PRVE_PLAT_ACCOUNT_TYPE", tostring(user_vo.plat_account_type))
	AdapterToLua:getInstance():setDataCache("PRVE_PLAT_FCM", tostring(user_vo.plat_fcm))
	AdapterToLua:getInstance():setDataCache("PRVE_PLAT_LOGIN_TIME", tostring(user_vo.plat_login_time))
	AdapterToLua:getInstance():setDataCache("PRVE_PLAT_SESSION_KEY", tostring(user_vo.plat_session_key))
end

function LoginController:StartOpenAnimation()
	GlobalEventSystem:Fire(LoginEventType.START_OPENING_ANIMATION)
end

function LoginController:EndOpenAnimation()
	self:DoOpenCreateRoleView()
end

function LoginController:OnRecvMainRoleInfo()
	AudioManager.Instance:StopMusic()
	if self.combine_select_role_view then
		self.combine_select_role_view:Close()
	end
	self:DoLogin()
end

function LoginController:OnConnectGameServer(is_succ)
	if is_succ then
		self:SendLoginReq()
	else
		Log("Login::game_server_connect_fail")
		self.game_server_connect_times = self.game_server_connect_times + 1
		if self.game_server_connect_times >= 5 then
			self:ReconnectFail()
			self:OpenFailAlert(Language.Login.ConnectGameServerFailTip)
		else
			local str = Language.Login.ReconnectGameServer .. self.game_server_connect_times
			Log(str)
			SysMsgCtrl.Instance:ErrorRemind(str, true)
			GameNet.Instance:AsyncConnectGameServer(3)
		end
	end
end

function LoginController:OnDisconnectGameServer(disconnect_reason,code)
	Log("Login::game_server_disconnect", disconnect_reason)
	if GameNet.DISCONNECT_REASON_SILENT_LOGIN == disconnect_reason then
		return
	end

	if self:GetIsReConnectIng() then
		self:StartLogin()
		return
	end
	
	if nil == self.disconnect_alert then
		self.disconnect_alert = Alert.New()
		self.disconnect_alert:SetModal(true)
		self.disconnect_alert:SetIsAnyClickClose(false)
	end
	local ok_callback = nil
	local dis_str = ""
	local ok_str = ""
	if disconnect_reason == GameNet.DISCONNECT_REASON_BE_DEL
	or disconnect_reason == GameNet.DISCONNECT_REASON_MULTI_LOGIN then
		ok_callback = function ()
			if AgentAdapter.OnClickRestartGame then
				AgentAdapter.OnClickRestartGame()
			else	
				ReStart()
			end
		end
		if disconnect_reason == GameNet.DISCONNECT_REASON_BE_DEL then
			dis_str = Language.Common.Disconnect2
		else
			dis_str = Language.Login.MultiLogin
		end
		ok_str = Language.Login.ReLogin
	else
		ok_callback = function ()
			-- AdapterToLua:getInstance():setDataCache("GUA_JI_TYPE", GuajiCache.guaji_type)
			AdapterToLua:getInstance():setDataCache("IS_RECONNECT_ING", "true")
		 	ReStart()
		end
		if disconnect_reason == GameNet.DISCONNECT_REASON_SERVER_UNKNOWN then
			dis_str = string.format(Language.Common.Disconnect3,code and code or "")
		elseif 	disconnect_reason == GameNet.DISCONNECT_REASON_RESTART then
			dis_str = Language.Common.Disconnect4
		elseif 	disconnect_reason == GameNet.DISCONNECT_REASON_CMD_CLOSE then
			dis_str = Language.Common.Disconnect5
		elseif 	disconnect_reason == GameNet.DISCONNECT_REASON_CONNECT_FAILED then
			dis_str = Language.Common.Disconnect6
		elseif 	disconnect_reason == GameNet.DISCONNECT_REASON_LOGIN_BACK then
			dis_str = Language.Common.Disconnect7
		elseif 	disconnect_reason == GameNet.DISCONNECT_REASON_NET_DISPOSE then
			dis_str = Language.Common.Disconnect8
		elseif disconnect_reason == GameNet.DISCONNECT_REASON_USEWAIGUA then	
			dis_str = Language.Common.Disconnect9
		elseif disconnect_reason == GameNet.DISCONNECT_REASON_CROSSLIMIT then	
			dis_str = Language.Common.Disconnect10
		else	
			dis_str = Language.Common.Disconnect
		end	
		ok_str = Language.Login.AutoReStart
	end
	self.disconnect_alert:SetLableString(dis_str)
	if disconnect_reason ~= GameNet.DISCONNECT_REASON_BE_DEL then
		self.disconnect_alert:SetOkString(ok_str)
	else
		self.disconnect_alert:SetOkString(ok_str .. "(" .. Auto_ReStart_Time .. ")")
		local cd = Auto_ReStart_Time
		local function cd_callback()
			cd = math.max(cd - 1, 0 )
			self.disconnect_alert:SetOkString(ok_str.. "(" .. cd .. ")")
		end
		CountDownManager.Instance:RemoveCountDown("re_start")
		CountDownManager.Instance:AddCountDown("re_start", cd_callback, ok_callback, nil, Auto_ReStart_Time, 1)
	end
	self.disconnect_alert:SetOkFunc(ok_callback)--muyi5242095
	self.disconnect_alert:Open()
	self.disconnect_alert:NoCloseButton()
	self.disconnect_alert:UseOne()
end

function LoginController:SendLoginReq()
	local user_vo = GameVoManager.Instance:GetUserVo()
	
	local protocol = ProtocolPool.Instance:GetProtocol(CSLoginReq)
	protocol.account = user_vo.plat_name
	protocol.password = user_vo.plat_pwd
	protocol.server_span_id =  user_vo.cross_server_id or user_vo.merge_id or user_vo.plat_server_id
	protocol.server_id =  user_vo.merge_id or user_vo.plat_server_id
	protocol:EncodeAndSend(GameNet.Instance:GetGameServerNetId())
	--print("请求登陆:",protocol.server_span_id,protocol.server_id)
	PlatformAdapter:SaveShareValue(AgentAdapter:GetPlatName() .. "last_login_server", user_vo.plat_server_id)
end

function LoginController:OnLoginFailAck(protocol)
	Log("登录失败", protocol.error_code)

	self:RemoveWaitingEffect()
	
	if 1 == protocol.error_code then
	elseif 2 == protocol.error_code then
		-- self:SendCreateAccountReq()
	elseif 3 == protocol.error_code then
	end

	-- enSuccess = 0,	     //操作成功
	-- enPasswdError =1,    //密码错误
	-- enNoAccount=2,       //没有这个账号
	-- enIsOnline =3,       //已经在线
	-- enServerBusy =4,     //服务器忙
	-- enServerClose =5,    //服务器没有开放 
	-- enSessionServerError =6 , //ssion服务器有问题，比如db没有连接好
	-- enServerNotExisting =7, //不存在这个服务器
	-- enFcmLimited =8 ,      //账户纳入防沉迷
	-- enSessionServerClose =9, //会话服务器处于关闭状态
	-- enDbServerClose =10,     //数据服务器处于关闭状态
	-- enLoginIpLimit = 11,     //登陆IP数量限制
	-- enIpError = 12,    //ip收到了限制
	-- enIpTooManyConnect = 13,    //ip连接了太多
	-- enIpMd5Error = 14,    //Md5计算错误
	-- enSignOutofDate = 15,    //发过来的时间已经过期了
	-- enTimeFormatError = 16,    //前面的格式错误
	-- enAccountSeal = 17,    //登陆过封停的账户
	--muyiqq5242095  
end

function LoginController:SendCreateAccountReq()
	Log("LoginController:SendCreateAccountReq")
	local user_vo = GameVoManager.Instance:GetUserVo()

	local protocol = ProtocolPool.Instance:GetProtocol(CSCreateAccountReq)
	protocol.account = user_vo.plat_name
	protocol.password = UtilEx:md5Data("000")
	protocol:EncodeAndSend(GameNet.Instance:GetGameServerNetId())
end

function LoginController:SendRoleListReq()
	Log("LoginController:SendRoleListReq")
	local protocol = ProtocolPool.Instance:GetProtocol(CSRoleListReq)
	protocol:EncodeAndSend()
end

function LoginController:OnRoleListAck(protocol)
	LogT("OnRoleListAck", protocol.role_count)
	local user_vo = GameVoManager.Instance:GetUserVo()

	if protocol.account_id > 0 then
		user_vo.account_id = protocol.account_id
		AdapterToLua:getInstance():setDataCache("PRVE_ACCOUNT_ID",user_vo.account_id)
	end

	local cs_info_str = AdapterToLua:getInstance():getDataCache("CROSS_SERVER_INFO")
	if nil ~= cs_info_str and "" ~= cs_info_str then
		local user_vo = GameVoManager.Instance:GetUserVo()
		user_vo.account_id = tonumber(AdapterToLua:getInstance():getDataCache("PRVE_ACCOUNT_ID")) or 0
		local last_role_id = tonumber(AdapterToLua:getInstance():getDataCache("SET_NOW_ROLE")) or 0
		user_vo:SetNowRole(last_role_id)
		self:DoLogin()
		self:SendLoginRoleReq()
		self:ReconnectSuccess()
		return
	end	

	if 0 == protocol.role_count then

		
		local sp_id = AgentAdapter:GetSpid()
		if self:IsLimitServerWithSpid(sp_id) then

		else	
			if IS_AUDIT_VERSION then -- 审核版直接创建角色
				self:RemoveWaitingEffect()
				self:RandomCreateRole()
			else	
				self:RemoveWaitingEffect()
				self:DoOpenCreateRoleView()
			end
		end	
		
	else
		user_vo:ClearRoleList()
		for i, v in ipairs(protocol.role_list) do
			user_vo:AddRole(v)
			if self.create_role == v.role_name then
				user_vo:SetNowRole(v.role_id)
			end
		end
		local is_reconnect_ing = AdapterToLua:getInstance():getDataCache("IS_RECONNECT_ING")
		local is_reselectrole = AdapterToLua:getInstance():getDataCache("IS_RESELECTROLE")
		local last_role_id = tonumber(AdapterToLua:getInstance():getDataCache("SET_NOW_ROLE")) or 0
		if is_reconnect_ing == "true" and last_role_id > 0 and is_reselectrole ~= "true" then
			user_vo:SetNowRole(last_role_id)
		end
		if (user_vo:GetNowRole() > 0 or is_reconnect_ing == "true") and is_reselectrole ~= "true" then
			if is_reconnect_ing == "true" then
				self:ReconnectSuccess()
			end
			self:DoLogin()
			self:SendLoginRoleReq()
		else
			if is_reselectrole == "true" then
				self:ReconnectSuccess()
			end
			self:RemoveWaitingEffect()
			self:DoOpenCombineSelectRoleView()
		end
	end

	--角色返回就播放创角声音

	local audio_res_path = ResPath.GetAudioBgResPath(AudioBg.CreateRole)
	AudioManager.Instance:PlayMusic(audio_res_path)
end

function LoginController:IsLimitServerWithSpid(spid)
	
	local user_vo = GameVoManager.Instance:GetUserVo()

	local limit_day = user_vo.create_role_limit_day
	if limit_day and limit_day > 0 then
		local now_server_time = GLOBAL_CONFIG.server_info.server_time + (Status.NowTime - GLOBAL_CONFIG.client_time)
		if 0 < user_vo.open_time then
			local time_space = now_server_time - user_vo.open_time
			local compare_time = limit_day * 24 * 3600  --开服限制天以上
			if time_space > compare_time then
				if nil == self.limit_alert then
					self.limit_alert = Alert.New("")
					self.limit_alert.zorder = COMMON_CONSTS.ZORDER_MAX
					self.limit_alert:SetModal(true)
					self.limit_alert:SetIsAnyClickClose(false)
					self.limit_alert:SetOkString(Language.Common.Confirm)
					self.limit_alert:SetCancelString(Language.Common.Cancel)
				end

				self.limit_alert:SetLableString(Language.Common.Disconnect11)
				self.limit_alert:Open()
				self.limit_alert:NoCloseButton()
				GameNet.Instance:DisconnectGameServer(GameNet.DISCONNECT_REASON_SILENT_LOGIN)
				return true
			end	
		end	
	end	

	return false
end

function LoginController:RandomCreateRole()
	local sex = math.random(0,1)
	local name = UserVo.RandomName(sex)
	local prof = math.random(1,3)
	self:SendCreateRole(name, prof, sex)
end	

function LoginController:DoLogin()
	self.login_state = LOGIN_STATE_LOADING
	self:RemoveWaitingEffect()
	self.loginview:OpenLoading()
end

function LoginController:SendLoginRoleReq()
	local default_index = 0
	if IS_AUDIT_VERSION then
		default_index = 10
	end	
	local protocol = ProtocolPool.Instance:GetProtocol(CSLoginRoleReq)
	protocol.account_id = GameVoManager.Instance:GetUserVo().account_id
	protocol.role_id = GameVoManager.Instance:GetUserVo().cur_role_id
	protocol.role_index = default_index
	protocol:EncodeAndSend()
end

function LoginController:DoOpenCreateRoleView()
	self.createroleview:Open()
	self.createroleview:OpenRoleCreate()
end

function LoginController:DoOpenCombineSelectRoleView()
	self.combine_select_role_view:Open()
	self.combine_select_role_view:OpenRoleSelect()
end

function LoginController:CreateRoleViewReturnLogin()
	GameNet.Instance:DisconnectGameServer(GameNet.DISCONNECT_REASON_SILENT_LOGIN)
	self.login_state = LOGIN_STATE_SERVER_LIST
	self.loginview:SetLoginVisible(true)
	if self.loginview.sec_server_list then
		self.loginview.sec_server_list:SelectIndex(1)
	end
	self.loginview:SelectTabCallback(1)
end

function LoginController:SendCreateRole(role_name, prof, sex)
	Log("Login::LoginController:SendCreateRole", role_name, prof, sex)
	self:ShowWaitingEffect()

	local mainrole_vo = GameVoManager.Instance:GetMainRoleVo()
	mainrole_vo.name = role_name

	local user_vo = GameVoManager.Instance:GetUserVo()
	local send_protocol = ProtocolPool.Instance:GetProtocol(CSCreateRoleReq)

	local merge_id = user_vo.merge_id or user_vo.plat_server_id
	self.create_role = "s" .. merge_id .. "." .. role_name
	send_protocol.name = self.create_role
	send_protocol.sex = sex
	send_protocol.prof = prof
	send_protocol.avatar = 0
	send_protocol.camp = 0
	send_protocol.spid = AgentAdapter:GetSpid()
	send_protocol.adid = 0
	send_protocol:EncodeAndSend()
end

function LoginController:SendDelRoleReq(role_id)
	Log("Login::LoginController:SendDelRoleReq", role_id)
	local protocol = ProtocolPool.Instance:GetProtocol(CSDelRoleReq)
	protocol.role_id = role_id
	protocol:EncodeAndSend()
end

function LoginController:TryEnterGameServer(role_vo)
	local user_vo = GameVoManager.Instance:GetUserVo()
	if role_vo then
		user_vo:SetNowRole(role_vo.role_id)
		self.send_role_req_flag = true
		self:SendLoginRoleReq()
		local mainrole_vo = GameVoManager.Instance:GetMainRoleVo()
		mainrole_vo.name = role_vo.role_name
	end
end

function LoginController:OnCreateRoleAck(protocol)
	LogT("====OnCreateRoleAck", protocol.role_id, protocol.result)
	if 0 == protocol.result then
		--self:SendRoleListReq()
		local user_vo = GameVoManager.Instance:GetUserVo()
		user_vo:SetIsCreatingRole(true)
		user_vo:SetNowRole(protocol.role_id)
		self:DoLogin()
		self:SendLoginRoleReq()
		if self.createroleview then
			self.createroleview:Close()
		end
		local mainrole_vo = GameVoManager.Instance:GetMainRoleVo()
		MainProber:Step2(MainProber.PHP_PUSH_CREATE_ROLE_END, MainProber.user_id, MainProber.server_id, mime.b64(mainrole_vo.name), user_vo.cur_role_id)

		if user_vo.fill_pay_url and user_vo.fill_pay_url ~= "" then
			if #user_vo:GetRoleList() == 0 then
				--请求元宝返还验证
				local role_vo =  GameVoManager.Instance:GetMainRoleVo()
				local key = "hdISla9sjXphPqEoE8lZcg=="
				local spid = AgentAdapter:GetSpid()
				local server_id = user_vo.real_server_id
				local user_id = AgentAdapter:GetPlatName() or ""
				local role_id = user_vo.cur_role_id
				local time = math.floor(TimeCtrl.Instance:GetServerTime())
				local sign = UtilEx:md5Data(spid .. server_id .. user_id .. role_id .. time .. key)
				local role_name = role_vo.name

				local url_str = user_vo.fill_pay_url .. "?" ..
									"spid=" .. spid ..
									"&server_id=" .. server_id ..
									"&user_id=" .. user_id ..
									"&role_id=" .. role_id ..
									"&time=" .. time ..
									"&sign=" .. sign ..
									"&role_name=" .. HttpClient:UrlEncode(role_name)
				HttpClient:Request(url_str, "")
			end	
		end	

	elseif 1 == protocol.result then
		SysMsgCtrl.Instance:ErrorRemind(Language.Login.InvalidName, true)
	elseif 4 == protocol.result then 
		SysMsgCtrl.Instance:ErrorRemind(Language.Login.OccupiedName, true)
	else
		self:RemoveWaitingEffect()
	end
end

function LoginController:OnDelRoleAck(protocol)
	LogT("====OnDelRoleAck", protocol.role_id, protocol.result)
	if 0 == protocol.result then
		local user_vo = GameVoManager.Instance:GetUserVo()
		user_vo:RemoveRole(protocol.role_id)
		if self.combine_select_role_view:IsOpen() then
			self.combine_select_role_view:UpdateRoleList()
		end
	end
end


function LoginController:OnDisconnectServerNotice(protocol)
	LogT("====OnDisconnectServerNotice", protocol.reason)
	if protocol.reason == CloseType.CT_ACTORDEL then
		-- GameNet.Instance:DisconnectLoginServer(GameNet.DISCONNECT_REASON_BE_DEL)
		GameNet.Instance:DisconnectGameServer(GameNet.DISCONNECT_REASON_BE_DEL)
	elseif protocol.reason == CloseType.CT_OTHERLOGIN then
		GameNet.Instance:DisconnectGameServer(GameNet.DISCONNECT_REASON_MULTI_LOGIN)
	elseif protocol.reason == CloseType.CT_USEWAIGUA then	
		GameNet.Instance:DisconnectGameServer(GameNet.DISCONNECT_REASON_USEWAIGUA)
	elseif protocol.reason == CloseType.CT_CROSSLIMIT then	
		GameNet.Instance:DisconnectGameServer(GameNet.DISCONNECT_REASON_CROSSLIMIT)
	else
		GameNet.Instance:DisconnectGameServer(GameNet.DISCONNECT_REASON_SERVER_UNKNOWN,protocol.reason)
	end
	
end

--进入跨服处理
function LoginController:OnCrossServerInfo(protocol)
	Log("====cross server info:",protocol.server_index,protocol.server_addr,protocol.server_port)

	local user_vo = GameVoManager.Instance:GetUserVo()
	local cross_params_format = "%s##%s##%s##%s##%s##%s##%s##%s##%s##%s##%s##%s"		
	local result = string.format(cross_params_format,
									protocol.server_index,"",protocol.server_addr,protocol.server_port,
									"","","","",
									"","","",user_vo.plat_server_id)
	--进行记录操作
	AdapterToLua:getInstance():setDataCache("CROSS_SERVER_INFO",result)
	AdapterToLua:getInstance():setDataCache("IS_RECONNECT_ING", "true")
	AdapterToLua:getInstance():setDataCache("IS_RESELECTROLE", "false")
	ReStart()
end	

--退出跨服处理
function LoginController:OnQuitCrossServerInfo(protocol)
	AdapterToLua:getInstance():setDataCache("IS_RECONNECT_ING", "true")
	AdapterToLua:getInstance():setDataCache("IS_RESELECTROLE", "false")
	ReStart()
end	

function LoginController:Reconnect()
	if self:GetIsReConnectIng() then
		local user_vo = GameVoManager.Instance:GetUserVo()
		user_vo.plat_name = AdapterToLua:getInstance():getDataCache("PRVE_ACCOUNT_NAME")
		user_vo.plat_server_id = tonumber(AdapterToLua:getInstance():getDataCache("PRVE_SRVER_ID")) or 0
		user_vo.merge_id = tonumber(AdapterToLua:getInstance():getDataCache("MERGE_ID")) or nil
		user_vo.real_server_id = user_vo.plat_server_id
		user_vo.plat_account_type = tonumber(AdapterToLua:getInstance():getDataCache("PRVE_PLAT_ACCOUNT_TYPE")) or 0
		user_vo.plat_fcm = tonumber(AdapterToLua:getInstance():getDataCache("PRVE_PLAT_FCM")) or 0
		user_vo.plat_login_time = tonumber(AdapterToLua:getInstance():getDataCache("PRVE_PLAT_LOGIN_TIME")) or 0
		user_vo.plat_session_key = AdapterToLua:getInstance():getDataCache("PRVE_PLAT_SESSION_KEY")
		user_vo.plat_server_name = AdapterToLua:getInstance():getDataCache("PRVE_SRVER_NAME") or ""
		user_vo.open_time = tonumber(AdapterToLua:getInstance():getDataCache("SERVER_OPENTIME")) or 0
		
		
		local cs_info_str = AdapterToLua:getInstance():getDataCache("CROSS_SERVER_INFO")
		if nil ~= cs_info_str and "" ~= cs_info_str then
			local list = Split(cs_info_str, "##")
			user_vo.cross_server_id = list[1]
			user_vo.plat_server_id = list[12]
			user_vo.real_server_id = user_vo.plat_server_id
		end
		user_vo.plat_is_verify = true
		Log("Loggin::start_reconnect plat_name:" .. user_vo.plat_name .. ",plat_server_id:" .. user_vo.plat_server_id)
		self.loginview:Reconnect()
	end
end

function LoginController:ReconnectSuccess()
	if self:GetIsReConnectIng() then
		Log("Loggin::reconnect_success")
		self:EndReconnect()
	end
end

function LoginController:ReconnectFail()
	if self:GetIsReConnectIng() then
		Log("Loggin::reconnect_fail")
		self.loginview:Open()
		self:AgentLoginViewOpen(false)
		self.loginview:SetLoginVisible(false)
		self:EndReconnect()
	end
end

function LoginController:EndReconnect()
	Log("Loggin::reconnect_end")
	AdapterToLua:getInstance():setDataCache("IS_RECONNECT_ING", "false")
	AdapterToLua:getInstance():setDataCache("IS_RESELECTROLE", "false")
	AdapterToLua:getInstance():setDataCache("CROSS_SERVER_INFO", "")
end

--是否在重连中
function LoginController:GetIsReConnectIng()
	local name = AdapterToLua:getInstance():getDataCache("PRVE_ACCOUNT_NAME")
	local is_reconnect_ing = AdapterToLua:getInstance():getDataCache("IS_RECONNECT_ING")
	return name ~= "" and is_reconnect_ing ~= "" and is_reconnect_ing == "true"
end

function LoginController:OpenFailAlert(str)
	self:RemoveWaitingEffect()
	
	GameNet.Instance:DisconnectGameServer(GameNet.DISCONNECT_REASON_CONNECT_FAILED)

	if nil == self.fail_alert then
		self.fail_alert = Alert.New()
		local function ok_callback()
			if nil ~= self.createroleview then
				self.createroleview:DeleteMe()
				self.createroleview = RoleCreateView.New()
			end
			self.agent_login_view:DeleteMe()
			self.agent_login_view = AgentLoginView.New()
			self:AgentLoginViewOpen(false)

			self.loginview:DeleteMe()
			self.loginview = LoginView.New()
			self.loginview:Open()
		end
		local function cancel_callback()
			AdapterToLua:endGame()
		end

		self.fail_alert = Alert.New("", ok_callback, cancel_callback)
		self.fail_alert.is_checkvisible_onopen = true  --在开头动画中隐藏面板时不应该出现
		self.fail_alert.zorder = COMMON_CONSTS.ZORDER_MAX
		self.fail_alert:SetModal(true)
		self.fail_alert:SetIsAnyClickClose(false)
		self.fail_alert:SetOkString(Language.Login.ReLogin)
		self.fail_alert:SetCancelString(Language.Login.EndGame)
	end

	self.fail_alert:SetLableString(str)
	self.fail_alert:Open()
	self.fail_alert:NoCloseButton()
end

function LoginController:ShowWaitingEffect(is_force_show)
	local is_show = is_force_show or (self.loginview:GetIsInSelectServerView() or self.createroleview:IsOpen())
	if nil == self.connecting_effect and is_show then
		-- local anim_path, anim_name = ResPath.GetEffectAnimPath(3091)
		-- self.connecting_effect = RenderUnit.CreateAnimSprite(anim_path, anim_name, 0.1, 15)
		-- local screen_w = HandleRenderUnit:GetWidth()
		-- local screen_h = HandleRenderUnit:GetHeight()
		-- self.connecting_effect:setPosition(screen_w / 2, screen_h / 2)
		-- HandleRenderUnit:GetCoreScene():addChildToRenderGroup(self.connecting_effect, GRQ_UI_UP)
	end
end

function LoginController:RemoveWaitingEffect()
	if nil ~= self.connecting_effect then
		self.connecting_effect:removeFromParent()
		self.connecting_effect = nil
	end
end

function LoginController:ExitReq()
	-- local send_protocol = ProtocolPool.Instance:GetProtocol(CSDisconnectReq)
	-- send_protocol:EncodeAndSend(GameNet.Instance:GetGameServerNetId())
end

function LoginController:OnOpenNoticeView()
	self.notice_view:Open()
end
